/**
 * Token signing functionality for Knock authentication
 */
import type { TokenEntity, TokenGrant, TokenGrantOptions } from "./userTokens.mjs";
/**
 * Options for signing a user token
 */
export interface SignUserTokenOptions {
    /**
     * The signing key to use (in PEM or base-64 encoded format)
     * If not provided, it will try to use process.env.KNOCK_SIGNING_KEY
     */
    signingKey?: string | undefined;
    /**
     * How long the token should be valid for in seconds
     * @default 3600 (1 hour)
     */
    expiresInSeconds?: number | undefined;
    /**
     * Grants to include in the token
     */
    grants?: TokenGrant[] | undefined;
}
/**
 * Prepare user token grants
 */
export declare function maybePrepareUserTokenGrants(grants: TokenGrant[] | undefined): Record<string, TokenGrant['grants']> | undefined;
/**
 * Helper function to build user token grants to pass to the `signUserToken` method.
 *
 * @param entity {TokenEntity} The type of entity to build a grant for
 * @param grants {TokenGrantOptions} A list of grants to give to the entity for the user
 *
 * @returns {TokenGrant} A single token grant that can be passed to the signUserToken function
 */
export declare function buildUserTokenGrant(entity: TokenEntity, grants: TokenGrantOptions): TokenGrant;
/**
 * Generate JWT for authenticating client-side requests (e.g. in-app feeds)
 * For more information, visit https://docs.knock.app/in-app-ui/security-and-authentication#authentication-with-enhanced-security-enabled
 *
 * @param userId {string} The ID of the user that needs a token, e.g. the user viewing an in-app feed.
 * @param options Optionally specify the signing key to use (in PEM or base-64 encoded format), and how long the token should be valid for in seconds
 * @returns {Promise<string>} A JWT token that can be used to authenticate requests to the Knock API (e.g. by passing into the <KnockFeedProvider /> component)
 */
export declare function signUserToken(userId: string, options?: SignUserTokenOptions): Promise<string>;
//# sourceMappingURL=tokenSigner.d.mts.map