import { APIResource } from "../core/resource.js";
import * as SharedAPI from "./shared.js";
import * as UsersAPI from "./users/users.js";
import { APIPromise } from "../core/api-promise.js";
import { RequestOptions } from "../internal/request-options.js";
export declare class Audiences extends APIResource {
    /**
     * Adds one or more members to the specified audience.
     *
     * @example
     * ```ts
     * const response = await client.audiences.addMembers('key', {
     *   members: [
     *     {
     *       tenant: 'ingen_isla_nublar',
     *       user: { id: 'dr_sattler' },
     *     },
     *   ],
     * });
     * ```
     */
    addMembers(key: string, body: AudienceAddMembersParams, options?: RequestOptions): APIPromise<string>;
    /**
     * Returns a paginated list of members for the specified audience.
     *
     * @example
     * ```ts
     * const response = await client.audiences.listMembers('key');
     * ```
     */
    listMembers(key: string, options?: RequestOptions): APIPromise<AudienceListMembersResponse>;
    /**
     * Removes one or more members from the specified audience.
     *
     * @example
     * ```ts
     * const response = await client.audiences.removeMembers(
     *   'key',
     *   { members: [{ user: {} }] },
     * );
     * ```
     */
    removeMembers(key: string, body: AudienceRemoveMembersParams, options?: RequestOptions): APIPromise<string>;
}
/**
 * An audience member.
 */
export interface AudienceMember {
    /**
     * The typename of the schema.
     */
    __typename: string;
    /**
     * Timestamp when the resource was created.
     */
    added_at: string;
    /**
     * A [User](/concepts/users) represents an individual in your system who can
     * receive notifications through Knock. Users are the most common recipients of
     * notifications and are always referenced by your internal identifier.
     */
    user: UsersAPI.User;
    /**
     * The ID for the user that you set when identifying them in Knock.
     */
    user_id: string;
    /**
     * The unique identifier for the tenant.
     */
    tenant?: string | null;
}
/**
 * A `204 No Content` response.
 */
export type AudienceAddMembersResponse = string;
/**
 * A paginated list of audience members.
 */
export interface AudienceListMembersResponse {
    /**
     * A list of audience members.
     */
    entries: Array<AudienceMember>;
    /**
     * Pagination information for a list of resources.
     */
    page_info: SharedAPI.PageInfo;
}
/**
 * A `204 No Content` response.
 */
export type AudienceRemoveMembersResponse = string;
export interface AudienceAddMembersParams {
    /**
     * A list of audience members to add.
     */
    members: Array<AudienceAddMembersParams.Member>;
}
export declare namespace AudienceAddMembersParams {
    /**
     * An audience member.
     */
    interface Member {
        /**
         * An object containing the user's ID.
         */
        user: Member.User;
        /**
         * The unique identifier for the tenant.
         */
        tenant?: string | null;
    }
    namespace Member {
        /**
         * An object containing the user's ID.
         */
        interface User {
            /**
             * The ID for the user that you set when identifying them in Knock.
             */
            id?: string;
        }
    }
}
export interface AudienceRemoveMembersParams {
    /**
     * A list of audience members to remove.
     */
    members: Array<AudienceRemoveMembersParams.Member>;
}
export declare namespace AudienceRemoveMembersParams {
    /**
     * An audience member.
     */
    interface Member {
        /**
         * An object containing the user's ID.
         */
        user: Member.User;
        /**
         * The unique identifier for the tenant.
         */
        tenant?: string | null;
    }
    namespace Member {
        /**
         * An object containing the user's ID.
         */
        interface User {
            /**
             * The ID for the user that you set when identifying them in Knock.
             */
            id?: string;
        }
    }
}
export declare namespace Audiences {
    export { type AudienceMember as AudienceMember, type AudienceAddMembersResponse as AudienceAddMembersResponse, type AudienceListMembersResponse as AudienceListMembersResponse, type AudienceRemoveMembersResponse as AudienceRemoveMembersResponse, type AudienceAddMembersParams as AudienceAddMembersParams, type AudienceRemoveMembersParams as AudienceRemoveMembersParams, };
}
//# sourceMappingURL=audiences.d.ts.map