import { APIResource } from "../../core/resource.mjs";
import { APIPromise } from "../../core/api-promise.mjs";
import { PagePromise, SlackChannelsCursor, type SlackChannelsCursorParams } from "../../core/pagination.mjs";
import { RequestOptions } from "../../internal/request-options.mjs";
export declare class Slack extends APIResource {
    /**
     * Check if a Slack channel is authenticated.
     *
     * @example
     * ```ts
     * const response = await client.providers.slack.checkAuth(
     *   'channel_id',
     *   { access_token_object: 'access_token_object' },
     * );
     * ```
     */
    checkAuth(channelID: string, query: SlackCheckAuthParams, options?: RequestOptions): APIPromise<SlackCheckAuthResponse>;
    /**
     * List Slack channels for a Slack workspace.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const slackListChannelsResponse of client.providers.slack.listChannels(
     *   'channel_id',
     *   { access_token_object: 'access_token_object' },
     * )) {
     *   // ...
     * }
     * ```
     */
    listChannels(channelID: string, query: SlackListChannelsParams, options?: RequestOptions): PagePromise<SlackListChannelsResponsesSlackChannelsCursor, SlackListChannelsResponse>;
    /**
     * Revoke access for a Slack channel.
     *
     * @example
     * ```ts
     * const response = await client.providers.slack.revokeAccess(
     *   'channel_id',
     *   { access_token_object: 'access_token_object' },
     * );
     * ```
     */
    revokeAccess(channelID: string, params: SlackRevokeAccessParams, options?: RequestOptions): APIPromise<SlackRevokeAccessResponse>;
}
export type SlackListChannelsResponsesSlackChannelsCursor = SlackChannelsCursor<SlackListChannelsResponse>;
/**
 * The response from a Slack auth check request.
 */
export interface SlackCheckAuthResponse {
    /**
     * A Slack connection object.
     */
    connection: SlackCheckAuthResponse.Connection;
}
export declare namespace SlackCheckAuthResponse {
    /**
     * A Slack connection object.
     */
    interface Connection {
        /**
         * Whether the Slack connection is valid.
         */
        ok: boolean;
        /**
         * The reason for the Slack connection if it is not valid.
         */
        reason?: string | null;
    }
}
/**
 * A Slack channel.
 */
export interface SlackListChannelsResponse {
    /**
     * A Slack channel ID from the Slack provider.
     */
    id: string;
    /**
     * The team ID that the Slack channel belongs to.
     */
    context_team_id: string;
    /**
     * Whether the Slack channel is an IM channel.
     */
    is_im: boolean;
    /**
     * Whether the Slack channel is private.
     */
    is_private: boolean;
    /**
     * Slack channel name.
     */
    name: string;
}
/**
 * A response indicating the operation was successful.
 */
export interface SlackRevokeAccessResponse {
    /**
     * OK response.
     */
    ok?: string;
}
export interface SlackCheckAuthParams {
    /**
     * A JSON encoded string containing the access token object reference.
     */
    access_token_object: string;
}
export interface SlackListChannelsParams extends SlackChannelsCursorParams {
    /**
     * A JSON encoded string containing the access token object reference.
     */
    access_token_object: string;
    query_options?: SlackListChannelsParams.QueryOptions;
}
export declare namespace SlackListChannelsParams {
    interface QueryOptions {
        /**
         * Paginate through collections of data by setting the cursor parameter to a
         * next_cursor attribute returned by a previous request's response_metadata.
         * Default value fetches the first "page" of the collection.
         */
        cursor?: string;
        /**
         * Set to true to exclude archived channels from the list.
         */
        exclude_archived?: boolean;
        /**
         * The maximum number of channels to return.
         */
        limit?: number;
        /**
         * Encoded team ID (T1234) to list channels in, required if org token is used.
         */
        team_id?: string;
        /**
         * Mix and match channel types by providing a comma-separated list of any
         * combination of public_channel, private_channel, mpim, im.
         */
        types?: string;
    }
}
export interface SlackRevokeAccessParams {
    /**
     * A JSON encoded string containing the access token object reference.
     */
    access_token_object: string;
}
export declare namespace Slack {
    export { type SlackCheckAuthResponse as SlackCheckAuthResponse, type SlackListChannelsResponse as SlackListChannelsResponse, type SlackRevokeAccessResponse as SlackRevokeAccessResponse, type SlackListChannelsResponsesSlackChannelsCursor as SlackListChannelsResponsesSlackChannelsCursor, type SlackCheckAuthParams as SlackCheckAuthParams, type SlackListChannelsParams as SlackListChannelsParams, type SlackRevokeAccessParams as SlackRevokeAccessParams, };
}
//# sourceMappingURL=slack.d.mts.map