import { APIResource } from "../../core/resource.js";
import * as ChannelDataAPI from "../recipients/channel-data.js";
import * as PreferencesAPI from "../recipients/preferences.js";
import * as BulkAPI from "./bulk.js";
import { Bulk, BulkDeleteParams, BulkSetParams } from "./bulk.js";
import { APIPromise } from "../../core/api-promise.js";
import { EntriesCursor, type EntriesCursorParams, PagePromise } from "../../core/pagination.js";
import { RequestOptions } from "../../internal/request-options.js";
export declare class Tenants extends APIResource {
    bulk: BulkAPI.Bulk;
    /**
     * List tenants for the current environment.
     *
     * @example
     * ```ts
     * // Automatically fetches more pages as needed.
     * for await (const tenant of client.tenants.list()) {
     *   // ...
     * }
     * ```
     */
    list(query?: TenantListParams | null | undefined, options?: RequestOptions): PagePromise<TenantsEntriesCursor, Tenant>;
    /**
     * Delete a tenant and all associated data. This operation cannot be undone.
     *
     * @example
     * ```ts
     * const tenant = await client.tenants.delete('id');
     * ```
     */
    delete(id: string, options?: RequestOptions): APIPromise<string>;
    /**
     * Get a tenant by ID.
     *
     * @example
     * ```ts
     * const tenant = await client.tenants.get('id');
     * ```
     */
    get(id: string, options?: RequestOptions): APIPromise<Tenant>;
    /**
     * Sets a tenant within an environment, performing an upsert operation. Any
     * existing properties will be merged with the incoming properties.
     *
     * @example
     * ```ts
     * const tenant = await client.tenants.set('id', {
     *   settings: {
     *     branding: {
     *       icon_url:
     *         'https://example.com/trex_silhouette_icon.png',
     *       logo_url: 'https://example.com/amber_fossil_logo.png',
     *       primary_color: '#DF1A22',
     *       primary_color_contrast: '#FFDE00',
     *     },
     *   },
     * });
     * ```
     */
    set(id: string, body: TenantSetParams, options?: RequestOptions): APIPromise<Tenant>;
}
export type TenantsEntriesCursor = EntriesCursor<Tenant>;
/**
 * An request to set a tenant inline.
 */
export type InlineTenantRequest = string | TenantRequest;
/**
 * A tenant entity.
 */
export interface Tenant {
    /**
     * The unique identifier for the tenant.
     */
    id: string;
    /**
     * The typename of the schema.
     */
    __typename: string;
    /**
     * An optional name for the tenant.
     */
    name?: string | null;
    /**
     * The settings for the tenant. Includes branding and preference set.
     */
    settings?: Tenant.Settings | null;
    [k: string]: unknown;
}
export declare namespace Tenant {
    /**
     * The settings for the tenant. Includes branding and preference set.
     */
    interface Settings {
        /**
         * The branding for the tenant.
         */
        branding?: Settings.Branding | null;
        /**
         * A preference set represents a specific set of notification preferences for a
         * recipient. A recipient can have multiple preference sets.
         */
        preference_set?: PreferencesAPI.PreferenceSet | null;
    }
    namespace Settings {
        /**
         * The branding for the tenant.
         */
        interface Branding {
            /**
             * The icon URL for the tenant. Must point to a valid image with an image MIME
             * type.
             */
            icon_url?: string | null;
            /**
             * The logo URL for the tenant. Must point to a valid image with an image MIME
             * type.
             */
            logo_url?: string | null;
            /**
             * The primary color for the tenant, provided as a hex value.
             */
            primary_color?: string | null;
            /**
             * The primary color contrast for the tenant, provided as a hex value.
             */
            primary_color_contrast?: string | null;
        }
    }
}
/**
 * A tenant to be set in the system. You can supply any additional properties on
 * the tenant object.
 */
export interface TenantRequest {
    /**
     * The unique identifier for the tenant.
     */
    id: string;
    /**
     * A request to set channel data for a type of channel inline.
     */
    channel_data?: ChannelDataAPI.InlineChannelDataRequest | null;
    /**
     * Inline set preferences for a recipient, where the key is the preference set id.
     */
    preferences?: PreferencesAPI.InlinePreferenceSetRequest | null;
    /**
     * The settings for the tenant. Includes branding and preference set.
     */
    settings?: TenantRequest.Settings;
    [k: string]: unknown;
}
export declare namespace TenantRequest {
    /**
     * The settings for the tenant. Includes branding and preference set.
     */
    interface Settings {
        /**
         * The branding for the tenant.
         */
        branding?: Settings.Branding;
        /**
         * A request to set a preference set for a recipient.
         */
        preference_set?: PreferencesAPI.PreferenceSetRequest | null;
    }
    namespace Settings {
        /**
         * The branding for the tenant.
         */
        interface Branding {
            /**
             * The icon URL for the tenant. Must point to a valid image with an image MIME
             * type.
             */
            icon_url?: string | null;
            /**
             * The logo URL for the tenant. Must point to a valid image with an image MIME
             * type.
             */
            logo_url?: string | null;
            /**
             * The primary color for the tenant, provided as a hex value.
             */
            primary_color?: string | null;
            /**
             * The primary color contrast for the tenant, provided as a hex value.
             */
            primary_color_contrast?: string | null;
        }
    }
}
/**
 * A `204 No Content` response.
 */
export type TenantDeleteResponse = string;
export interface TenantListParams extends EntriesCursorParams {
    /**
     * Filter tenants by name.
     */
    name?: string;
    /**
     * Filter tenants by ID.
     */
    tenant_id?: string;
}
export interface TenantSetParams {
    /**
     * A request to set channel data for a type of channel inline.
     */
    channel_data?: ChannelDataAPI.InlineChannelDataRequest | null;
    /**
     * The settings for the tenant. Includes branding and preference set.
     */
    settings?: TenantSetParams.Settings;
    [k: string]: unknown;
}
export declare namespace TenantSetParams {
    /**
     * The settings for the tenant. Includes branding and preference set.
     */
    interface Settings {
        /**
         * The branding for the tenant.
         */
        branding?: Settings.Branding;
        /**
         * A request to set a preference set for a recipient.
         */
        preference_set?: PreferencesAPI.PreferenceSetRequest | null;
    }
    namespace Settings {
        /**
         * The branding for the tenant.
         */
        interface Branding {
            /**
             * The icon URL for the tenant. Must point to a valid image with an image MIME
             * type.
             */
            icon_url?: string | null;
            /**
             * The logo URL for the tenant. Must point to a valid image with an image MIME
             * type.
             */
            logo_url?: string | null;
            /**
             * The primary color for the tenant, provided as a hex value.
             */
            primary_color?: string | null;
            /**
             * The primary color contrast for the tenant, provided as a hex value.
             */
            primary_color_contrast?: string | null;
        }
    }
}
export declare namespace Tenants {
    export { type InlineTenantRequest as InlineTenantRequest, type Tenant as Tenant, type TenantRequest as TenantRequest, type TenantDeleteResponse as TenantDeleteResponse, type TenantsEntriesCursor as TenantsEntriesCursor, type TenantListParams as TenantListParams, type TenantSetParams as TenantSetParams, };
    export { Bulk as Bulk, type BulkDeleteParams as BulkDeleteParams, type BulkSetParams as BulkSetParams };
}
//# sourceMappingURL=tenants.d.ts.map